@@ -26,16 +26,16 @@
 
 
 static Eina_Inlist *
-get_layer_objects_last(Evas_Layer *l)
+get_layer_objects(Evas_Layer *l)
 {
    if( !l || !l->objects ) return NULL;
 
-   return (EINA_INLIST_GET(l->objects))->last;
+   return (EINA_INLIST_GET(l->objects));
 }
 
 /* evas internal stuff */
 Evas_Object *
-evas_object_new(void)
+evas_object_new(Evas *e)
 {
    Evas_Object *obj;
 
@@ -54,6 +54,7 @@ evas_object_free(Evas_Object *obj, int clean_layer)
 {
    int was_smart_child = 0;
 
+   evas_object_map_set(obj, NULL);
    evas_object_grabs_cleanup(obj);
    evas_object_intercept_cleanup(obj);
    if (obj->smart.parent) was_smart_child = 1;
@@ -82,14 +83,14 @@ void
 evas_object_change(Evas_Object *obj)
 {
    Eina_List *l;
-   Evas_Object *data;
+   Evas_Object *obj2;
 
    obj->layer->evas->changed = 1;
    if (obj->changed) return;
+//   obj->changed = 1;
    evas_render_object_recalc(obj);
    /* set changed flag on all objects this one clips too */
-   EINA_LIST_FOREACH(obj->clip.clipees, l, data)
-     evas_object_change(data);
+   EINA_LIST_FOREACH(obj->clip.clipees, l, obj2) evas_object_change(obj2);
    if (obj->smart.parent) evas_object_change(obj->smart.parent);
 }
 
@@ -172,39 +173,49 @@ void
 evas_object_render_pre_prev_cur_add(Eina_Array *rects, Evas_Object *obj)
 {
    evas_add_rect(rects,
-		 obj->cur.geometry.x,
-		 obj->cur.geometry.y,
-		 obj->cur.geometry.w,
-		 obj->cur.geometry.h);
+                 obj->cur.cache.clip.x,
+                 obj->cur.cache.clip.y,
+                 obj->cur.cache.clip.w,
+                 obj->cur.cache.clip.h);
+   evas_add_rect(rects,
+                 obj->prev.cache.clip.x,
+                 obj->prev.cache.clip.y,
+                 obj->prev.cache.clip.w,
+                 obj->prev.cache.clip.h);
+/*        
+        evas_add_rect(rects,
+                      obj->cur.geometry.x,
+                      obj->cur.geometry.y,
+                      obj->cur.geometry.w,
+                      obj->cur.geometry.h);
 ////	    obj->cur.cache.geometry.x,
 ////	    obj->cur.cache.geometry.y,
 ////	    obj->cur.cache.geometry.w,
 ////	    obj->cur.cache.geometry.h);
-   evas_add_rect(rects,
-		 obj->prev.geometry.x,
-		 obj->prev.geometry.y,
-		 obj->prev.geometry.w,
-		 obj->prev.geometry.h);
+        evas_add_rect(rects,
+                      obj->prev.geometry.x,
+                      obj->prev.geometry.y,
+                      obj->prev.geometry.w,
+                      obj->prev.geometry.h);
 ////	    obj->prev.cache.geometry.x,
 ////	    obj->prev.cache.geometry.y,
 ////	    obj->prev.cache.geometry.w,
 ////	    obj->prev.cache.geometry.h);
+*/
 }
 
 void
 evas_object_clip_changes_clean(Evas_Object *obj)
 {
-   Evas_Rectangle *r;
+   Eina_Rectangle *r;
 
-   EINA_LIST_FREE(obj->clip.changes, r)
-     eina_mempool_free(_evas_rectangle_mp, r);
+   EINA_LIST_FREE(obj->clip.changes, r) eina_rectangle_free(r);
 }
 
-
 void
 evas_object_render_pre_effect_updates(Eina_Array *rects, Evas_Object *obj, int is_v, int was_v)
 {
-   Evas_Rectangle *r;
+   Eina_Rectangle *r;
    Evas_Object *clipper;
    Eina_List *l;
    unsigned int i;
@@ -288,7 +299,7 @@ evas_object_render_pre_effect_updates(Eina_Array *rects, Evas_Object *obj, int i
 
  end:
    EINA_ARRAY_ITER_NEXT(rects, i, r, it)
-     eina_mempool_free(_evas_rectangle_mp, r);
+     eina_rectangle_free(r);
    eina_array_clean(rects);
 }
 
@@ -338,12 +349,6 @@ evas_object_was_inside(Evas_Object *obj, Evas_Coord x, Evas_Coord y)
 }
 /* routines apps will call */
 
-/**
- * @defgroup Evas_Object_Group Generic Object Functions
- *
- * Functions that manipulate generic evas objects.
- */
-
 /**
  * Deletes the given evas object and frees its memory.
  *
@@ -352,7 +357,7 @@ evas_object_was_inside(Evas_Object *obj, Evas_Coord x, Evas_Coord y)
  * also called.
  *
  * @param   obj The given evas object.
- * @ingroup Evas_Object_Group
+ * @ingroup Evas_Object_Group_Basic
  */
 EAPI void
 evas_object_del(Evas_Object *obj)
@@ -363,7 +368,14 @@ evas_object_del(Evas_Object *obj)
 
    if (obj->delete_me) return;
 
+#ifdef EVAS_FRAME_QUEUING
+   evas_common_frameq_flush();
+#endif
+
+   _evas_object_event_new();
+
    evas_object_event_callback_call(obj, EVAS_CALLBACK_DEL, NULL);
+   _evas_post_event_callback_call(obj->layer->evas);
    if (obj->name) evas_object_name_set(obj, NULL);
    if (!obj->layer)
      {
@@ -374,7 +386,9 @@ evas_object_del(Evas_Object *obj)
      {
 	obj->focused = 0;
 	obj->layer->evas->focused = NULL;
+	_evas_object_event_new();
 	evas_object_event_callback_call(obj, EVAS_CALLBACK_FOCUS_OUT, NULL);
+        _evas_post_event_callback_call(obj->layer->evas);
      }
    obj->layer->evas->pointer.mouse_grabbed -= obj->mouse_grabbed;
    obj->mouse_grabbed = 0;
@@ -383,7 +397,10 @@ evas_object_del(Evas_Object *obj)
    while (obj->clip.clipees) evas_object_clip_unset(obj->clip.clipees->data);
    if (obj->cur.clipper) evas_object_clip_unset(obj);
    if (obj->smart.smart) evas_object_smart_del(obj);
+   evas_object_map_set(obj, NULL);
+   _evas_object_event_new();
    evas_object_event_callback_call(obj, EVAS_CALLBACK_FREE, NULL);
+   _evas_post_event_callback_call(obj->layer->evas);
    evas_object_smart_cleanup(obj);
    obj->delete_me = 1;
    evas_object_change(obj);
@@ -394,7 +411,7 @@ evas_object_del(Evas_Object *obj)
  * @param   obj The given evas object.
  * @param   x   X position to move the object to, in canvas units.
  * @param   y   Y position to move the object to, in canvas units.
- * @ingroup Evas_Object_Group
+ * @ingroup Evas_Object_Group_Basic
  */
 EAPI void
 evas_object_move(Evas_Object *obj, Evas_Coord x, Evas_Coord y)
@@ -468,7 +485,24 @@ evas_object_move(Evas_Object *obj, Evas_Coord x, Evas_Coord y)
  * @param   obj The given evas object.
  * @param   w   The new width of the evas object.
  * @param   h   The new height of the evas object.
- * @ingroup Evas_Object_Group
+ *
+ * @note Be aware that resizing an object changes its drawing area,
+ *       but that does imply the object is rescaled! For instance,
+ *       images are filled inside their drawing area using the
+ *       specifications of evas_object_image_fill_set(), thus to scale
+ *       the image to match exactly your drawing area, you need to
+ *       change the evas_object_image_fill_set() as well. Consider the
+ *       following example:
+ *       @code
+ *       // rescale image to fill exactly its area without tiling:
+ *       evas_object_resize(img, w, h);
+ *       evas_object_image_fill_set(img, 0, 0, w, h);
+ *       @endcode
+ *       This is more evident in images, but text, textblock, lines
+ *       and polygons will behave similarly. Check their specific APIs
+ *       to know how to achive your desired behavior.
+ *
+ * @ingroup Evas_Object_Group_Basic
  */
 EAPI void
 evas_object_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h)
@@ -479,7 +513,7 @@ evas_object_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h)
    return;
    MAGIC_CHECK_END();
    if (obj->delete_me) return;
-   if (w < 0.0) w = 0.0; if (h < 0.0) h = 0.0;
+   if (w < 0) w = 0; if (h < 0) h = 0;
    if (evas_object_intercept_call_resize(obj, w, h)) return;
 #ifdef FORWARD_NOOP_RESIZES_TO_SMART_OBJS
    if (obj->smart.smart)
@@ -511,14 +545,18 @@ evas_object_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h)
 					      obj->layer->evas->pointer.x,
 					      obj->layer->evas->pointer.y, 1, 1);
      }
+
    obj->cur.geometry.w = w;
    obj->cur.geometry.h = h;
 ////   obj->cur.cache.geometry.validity = 0;
    evas_object_change(obj);
    evas_object_clip_dirty(obj);
-   evas_object_recalc_clippees(obj);
+   /* NB: evas_object_recalc_clippees was here previously ( < 08/07/2009) */
    if (obj->layer->evas->events_frozen <= 0)
      {
+        /* NB: If this creates glitches on screen then move to above position */
+        evas_object_recalc_clippees(obj);
+
 	//   if (obj->func->coords_recalc) obj->func->coords_recalc(obj);
 	if (!pass)
 	  {
@@ -554,7 +592,7 @@ evas_object_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h)
  *              object.
  * @param   h   Pointer to an integer in which to store the height of the
  *              object.
- * @ingroup Evas_Object_Group
+ * @ingroup Evas_Object_Group_Basic
  */
 EAPI void
 evas_object_geometry_get(const Evas_Object *obj, Evas_Coord *x, Evas_Coord *y, Evas_Coord *w, Evas_Coord *h)
@@ -575,7 +613,7 @@ evas_object_geometry_get(const Evas_Object *obj, Evas_Coord *x, Evas_Coord *y, E
 }
 
 /**
- * @addtogroup Evas_Object_Group
+ * @addtogroup Evas_Object_Group_Size_Hints
  * @{
  */
 
@@ -583,7 +621,7 @@ static void
 _evas_object_size_hint_alloc(Evas_Object *obj)
 {
    if (obj->size_hints) return;
-   
+
    obj->size_hints = calloc(1, sizeof(Evas_Size_Hints));
    obj->size_hints->max.w = -1;
    obj->size_hints->max.h = -1;
@@ -1014,7 +1052,7 @@ evas_object_size_hint_padding_set(Evas_Object *obj, Evas_Coord l, Evas_Coord r,
 /**
  * Makes the given evas object visible.
  * @param   obj The given evas object.
- * @ingroup Evas_Object_Group
+ * @ingroup Evas_Object_Group_Basic
  */
 EAPI void
 evas_object_show(Evas_Object *obj)
@@ -1063,7 +1101,13 @@ evas_object_show(Evas_Object *obj)
 /**
  * Makes the given evas object invisible.
  * @param   obj The given evas object.
- * @ingroup Evas_Object_Group
+ *
+ * @note the hidden objects will not be checked for changes and will
+ *       not catch events. That is, they are much ligher than an
+ *       object that is invisible due indirect effects, such as
+ *       clipped or out-of-viewport.
+ *
+ * @ingroup Evas_Object_Group_Basic
  */
 EAPI void
 evas_object_hide(Evas_Object *obj)
@@ -1122,7 +1166,9 @@ evas_object_hide(Evas_Object *obj)
                   if (obj->mouse_in)
                     {
                        Evas_Event_Mouse_Out ev;
-                       
+
+		       _evas_object_event_new();
+
                        obj->mouse_in = 0;
                        ev.buttons = obj->layer->evas->pointer.button;
                        ev.output.x = obj->layer->evas->pointer.x;
@@ -1132,7 +1178,10 @@ evas_object_hide(Evas_Object *obj)
                        ev.data = NULL;
                        ev.modifiers = &(obj->layer->evas->modifiers);
                        ev.locks = &(obj->layer->evas->locks);
+		       ev.timestamp = obj->layer->evas->last_timestamp;
+		       ev.event_flags = EVAS_EVENT_FLAG_NONE;
                        evas_object_event_callback_call(obj, EVAS_CALLBACK_MOUSE_OUT, &ev);
+                       _evas_post_event_callback_call(obj->layer->evas);
 		    }
 	       }
 	  }
@@ -1151,9 +1200,9 @@ evas_object_hide(Evas_Object *obj)
  * Retrieves whether or not the given evas object is visible.
  * @param   obj The given evas object.
  * @return  @c 1 if the object is visible.  @c 0 otherwise.
- * @ingroup Evas_Object_Group
+ * @ingroup Evas_Object_Group_Basic
  */
-EAPI Evas_Bool
+EAPI Eina_Bool
 evas_object_visible_get(const Evas_Object *obj)
 {
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
@@ -1170,7 +1219,7 @@ evas_object_visible_get(const Evas_Object *obj)
  * @param g   The green component of the given colour.
  * @param b   The blue component of the given colour.
  * @param a   The alpha component of the given colour.
- * @ingroup Evas_Object_Group
+ * @ingroup Evas_Object_Group_Basic
  */
 EAPI void
 evas_object_color_set(Evas_Object *obj, int r, int g, int b, int a)
@@ -1217,7 +1266,7 @@ evas_object_color_set(Evas_Object *obj, int r, int g, int b, int a)
  *              the colour.
  * @param   a   Pointer to an integer in which to store the alpha component of
  *              the colour.
- * @ingroup Evas_Object_Group
+ * @ingroup Evas_Object_Group_Basic
  */
 EAPI void
 evas_object_color_get(const Evas_Object *obj, int *r, int *g, int *b, int *a)
@@ -1238,13 +1287,14 @@ evas_object_color_get(const Evas_Object *obj, int *r, int *g, int *b, int *a)
 }
 
 /**
- * Sets whether or not the given evas object is to be drawn anti_aliased.
+ * Sets whether or not the given evas object is to be drawn anti-aliased.
+ *
  * @param   obj The given evas object.
- * @param   anti_alias. 1 if the object is to be anti_aliased, 0 otherwise.
- * @ingroup Evas_Object_Group
+ * @param   anti_alias 1 if the object is to be anti_aliased, 0 otherwise.
+ * @ingroup Evas_Object_Group_Extras
  */
 EAPI void
-evas_object_anti_alias_set(Evas_Object *obj, Evas_Bool anti_alias)
+evas_object_anti_alias_set(Evas_Object *obj, Eina_Bool anti_alias)
 {
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
@@ -1261,9 +1311,9 @@ evas_object_anti_alias_set(Evas_Object *obj, Evas_Bool anti_alias)
  * Retrieves whether or not the given evas object is to be drawn anti_aliased.
  * @param   obj The given evas object.
  * @return  @c 1 if the object is to be anti_aliased.  @c 0 otherwise.
- * @ingroup Evas_Object_Group
+ * @ingroup Evas_Object_Group_Extras
  */
-EAPI Evas_Bool
+EAPI Eina_Bool
 evas_object_anti_alias_get(const Evas_Object *obj)
 {
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
@@ -1275,9 +1325,10 @@ evas_object_anti_alias_get(const Evas_Object *obj)
 
 /**
  * Sets the scaling factor for an evas object. Does not affect all objects.
+ *
  * @param   obj The given evas object.
- * @param   scale. The scaling factor. 1.0 == none.
- * @ingroup Evas_Object_Group
+ * @param   scale The scaling factor. 1.0 == none.
+ * @ingroup Evas_Object_Group_Extras
  */
 EAPI void
 evas_object_scale_set(Evas_Object *obj, double scale)
@@ -1298,7 +1349,8 @@ evas_object_scale_set(Evas_Object *obj, double scale)
  * Retrieves the scaling factor for the given evas object.
  * @param   obj The given evas object.
  * @return  The scaling factor.
- * @ingroup Evas_Object_Group
+ *
+ * @ingroup Evas_Object_Group_Extras
  */
 EAPI double
 evas_object_scale_get(const Evas_Object *obj)
@@ -1312,9 +1364,11 @@ evas_object_scale_get(const Evas_Object *obj)
 
 /**
  * Sets the color_space to be used for linear interpolation of colors.
+ *
  * @param   obj The given evas object.
- * @param   color_space, one of EVAS_COLOR_SPACE_ARGB or EVAS_COLOR_SPACE_AHSV.
- * @ingroup Evas_Object_Group
+ * @param   color_space one of EVAS_COLOR_SPACE_ARGB or EVAS_COLOR_SPACE_AHSV.
+ *
+ * @ingroup Evas_Object_Group_Extras
  */
 EAPI void
 evas_object_color_interpolation_set(Evas_Object *obj, int color_space)
@@ -1324,7 +1378,7 @@ evas_object_color_interpolation_set(Evas_Object *obj, int color_space)
    MAGIC_CHECK_END();
    if (obj->delete_me) return;
    if (obj->cur.interpolation_color_space == color_space)
-   	return;
+	return;
    obj->cur.interpolation_color_space = color_space;
    evas_object_change(obj);
 }
@@ -1334,7 +1388,7 @@ evas_object_color_interpolation_set(Evas_Object *obj, int color_space)
  * Retrieves the current value of the color space used for linear interpolation.
  * @param   obj The given evas object.
  * @return  @c EVAS_COLOR_SPACE_ARGB or EVAS_COLOR_SPACE_AHSV.
- * @ingroup Evas_Object_Group
+ * @ingroup Evas_Object_Group_Extras
  */
 EAPI int
 evas_object_color_interpolation_get(const Evas_Object *obj)
@@ -1350,7 +1404,7 @@ evas_object_color_interpolation_get(const Evas_Object *obj)
  * Sets the render_op to be used for rendering the evas object.
  * @param   obj The given evas object.
  * @param   render_op one of the Evas_Render_Op values.
- * @ingroup Evas_Object_Group
+ * @ingroup Evas_Object_Group_Extras
  */
 EAPI void
 evas_object_render_op_set(Evas_Object *obj, Evas_Render_Op render_op)
@@ -1360,7 +1414,7 @@ evas_object_render_op_set(Evas_Object *obj, Evas_Render_Op render_op)
    MAGIC_CHECK_END();
    if (obj->delete_me) return;
    if ((Evas_Render_Op)obj->cur.render_op == render_op)
-   	return;
+	return;
    obj->cur.render_op = render_op;
    evas_object_change(obj);
 }
@@ -1370,7 +1424,7 @@ evas_object_render_op_set(Evas_Object *obj, Evas_Render_Op render_op)
  * Retrieves the current value of the operation used for rendering the evas object.
  * @param   obj The given evas object.
  * @return  one of the enumerated values in Evas_Render_Op.
- * @ingroup Evas_Object_Group
+ * @ingroup Evas_Object_Group_Extras
  */
 EAPI Evas_Render_Op
 evas_object_render_op_get(const Evas_Object *obj)
@@ -1386,7 +1440,7 @@ evas_object_render_op_get(const Evas_Object *obj)
  * Retrieves the evas that the given evas object is on.
  * @param   obj The given evas object.
  * @return  The evas that the object is on.
- * @ingroup Evas_Object_Group
+ * @ingroup Evas_Object_Group_Basic
  */
 EAPI Evas *
 evas_object_evas_get(const Evas_Object *obj)
@@ -1394,22 +1448,27 @@ evas_object_evas_get(const Evas_Object *obj)
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return NULL;
    MAGIC_CHECK_END();
-   if (obj->delete_me) return 0;
+   if (obj->delete_me) return NULL;
    return obj->layer->evas;
 }
 
 /**
- * @addtogroup Evas_Object_Group
+ * @addtogroup Evas_Object_Group_Find
  * @{
  */
 
 /**
- * To be documented.
- *
- * FIXME: To be fixed.
+ * Retrieves the top object at the given position (x,y)
+ * @param   e The given evas object.
+ * @param   x The horizontal coordinate
+ * @param   y The vertical coordinate
+ * @param   include_pass_events_objects Boolean Flag to include or not
+ * pass events objects
+ * @param   include_hidden_objects Boolean Flag to include or not hidden objects
+ * @return  The evas object that is over all others objects at the given position.
  */
 EAPI Evas_Object *
-evas_object_top_at_xy_get(const Evas *e, Evas_Coord x, Evas_Coord y, Evas_Bool include_pass_events_objects, Evas_Bool include_hidden_objects)
+evas_object_top_at_xy_get(const Evas *e, Evas_Coord x, Evas_Coord y, Eina_Bool include_pass_events_objects, Eina_Bool include_hidden_objects)
 {
    Evas_Layer *lay;
    int xx, yy;
@@ -1425,7 +1484,7 @@ evas_object_top_at_xy_get(const Evas *e, Evas_Coord x, Evas_Coord y, Evas_Bool i
      {
 	Evas_Object *obj;
 
-	EINA_INLIST_REVERSE_FOREACH(get_layer_objects_last(lay), obj)
+	EINA_INLIST_REVERSE_FOREACH(get_layer_objects(lay), obj)
 	  {
 	     if (obj->delete_me) continue;
 	     if ((!include_pass_events_objects) && (evas_event_passes_through(obj))) continue;
@@ -1440,9 +1499,10 @@ evas_object_top_at_xy_get(const Evas *e, Evas_Coord x, Evas_Coord y, Evas_Bool i
 }
 
 /**
- * To be documented.
- *
- * FIXME: To be fixed.
+ * Retrieves the top object at mouse pointer position
+ * @param   e The given evas object.
+ * @return The evas object that is over all others objects at the
+ * pointer position.
  */
 EAPI Evas_Object *
 evas_object_top_at_pointer_get(const Evas *e)
@@ -1452,12 +1512,19 @@ evas_object_top_at_pointer_get(const Evas *e)
 }
 
 /**
- * To be documented.
+ * Retrieves the top object in the given rectangle region
+ * @param   e The given evas object.
+ * @param   x The horizontal coordinate.
+ * @param   y The vertical coordinate.
+ * @param   w The width size.
+ * @param   h The height size.
+ * @param   include_pass_events_objects Boolean Flag to include or not pass events objects
+ * @param   include_hidden_objects Boolean Flag to include or not hidden objects
+ * @return  The evas object that is over all others objects at the pointer position.
  *
- * FIXME: To be fixed.
  */
 EAPI Evas_Object *
-evas_object_top_in_rectangle_get(const Evas *e, Evas_Coord x, Evas_Coord y, Evas_Coord w, Evas_Coord h, Evas_Bool include_pass_events_objects, Evas_Bool include_hidden_objects)
+evas_object_top_in_rectangle_get(const Evas *e, Evas_Coord x, Evas_Coord y, Evas_Coord w, Evas_Coord h, Eina_Bool include_pass_events_objects, Eina_Bool include_hidden_objects)
 {
    Evas_Layer *lay;
    int xx, yy, ww, hh;
@@ -1479,7 +1546,7 @@ evas_object_top_in_rectangle_get(const Evas *e, Evas_Coord x, Evas_Coord y, Evas
      {
 	Evas_Object *obj;
 
-	EINA_INLIST_REVERSE_FOREACH(get_layer_objects_last(lay), obj)
+	EINA_INLIST_REVERSE_FOREACH(get_layer_objects(lay), obj)
 	  {
 	     if (obj->delete_me) continue;
 	     if ((!include_pass_events_objects) && (evas_event_passes_through(obj))) continue;
@@ -1494,12 +1561,18 @@ evas_object_top_in_rectangle_get(const Evas *e, Evas_Coord x, Evas_Coord y, Evas
 }
 
 /**
- * To be documented.
+ * Retrieves the objects at the given position
+ * @param   e The given evas object.
+ * @param   x The horizontal coordinate.
+ * @param   y The vertical coordinate.
+ * @param include_pass_events_objects Boolean Flag to include or not
+ * pass events objects
+ * @param   include_hidden_objects Boolean Flag to include or not hidden objects
+ * @return  The list of evas objects at the pointer position.
  *
- * FIXME: To be fixed.
  */
 EAPI Eina_List *
-evas_objects_at_xy_get(const Evas *e, Evas_Coord x, Evas_Coord y, Evas_Bool include_pass_events_objects, Evas_Bool include_hidden_objects)
+evas_objects_at_xy_get(const Evas *e, Evas_Coord x, Evas_Coord y, Eina_Bool include_pass_events_objects, Eina_Bool include_hidden_objects)
 {
    Eina_List *in = NULL;
    Evas_Layer *lay;
@@ -1516,7 +1589,7 @@ evas_objects_at_xy_get(const Evas *e, Evas_Coord x, Evas_Coord y, Evas_Bool incl
      {
 	Evas_Object *obj;
 
-	EINA_INLIST_REVERSE_FOREACH(get_layer_objects_last(lay), obj)
+	EINA_INLIST_REVERSE_FOREACH(get_layer_objects(lay), obj)
 	  {
 	     if (obj->delete_me) continue;
 	     if ((!include_pass_events_objects) && (evas_event_passes_through(obj))) continue;
@@ -1535,8 +1608,20 @@ evas_objects_at_xy_get(const Evas *e, Evas_Coord x, Evas_Coord y, Evas_Bool incl
  *
  * FIXME: To be fixed.
  */
+/**
+ * Retrieves the objects in the given rectangle region
+ * @param   e The given evas object.
+ * @param   x The horizontal coordinate.
+ * @param   y The vertical coordinate.
+ * @param   w The width size.
+ * @param   h The height size.
+ * @param   include_pass_events_objects Boolean Flag to include or not pass events objects
+ * @param   include_hidden_objects Boolean Flag to include or not hidden objects
+ * @return  The list of evas object in the rectangle region.
+ *
+ */
 EAPI Eina_List *
-evas_objects_in_rectangle_get(const Evas *e, Evas_Coord x, Evas_Coord y, Evas_Coord w, Evas_Coord h, Evas_Bool include_pass_events_objects, Evas_Bool include_hidden_objects)
+evas_objects_in_rectangle_get(const Evas *e, Evas_Coord x, Evas_Coord y, Evas_Coord w, Evas_Coord h, Eina_Bool include_pass_events_objects, Eina_Bool include_hidden_objects)
 {
    Eina_List *in = NULL;
    Evas_Layer *lay;
@@ -1559,7 +1644,7 @@ evas_objects_in_rectangle_get(const Evas *e, Evas_Coord x, Evas_Coord y, Evas_Co
      {
 	Evas_Object *obj;
 
-	EINA_INLIST_REVERSE_FOREACH(get_layer_objects_last(lay), obj)
+	EINA_INLIST_REVERSE_FOREACH(get_layer_objects(lay), obj)
 	  {
 	     if (obj->delete_me) continue;
 	     if ((!include_pass_events_objects) && (evas_event_passes_through(obj))) continue;
@@ -1573,10 +1658,15 @@ evas_objects_in_rectangle_get(const Evas *e, Evas_Coord x, Evas_Coord y, Evas_Co
    return in;
 }
 
+/**
+ * @}
+ */
+
 /**
  * Retrieves the name of the type of the given evas object.
  * @param   obj The given object.
  * @return  The name.
+ * @ingroup Evas_Object_Group_Basic
  */
 EAPI const char *
 evas_object_type_get(const Evas_Object *obj)
@@ -1593,9 +1683,10 @@ evas_object_type_get(const Evas_Object *obj)
  * @param obj The given object.
  * @param precise wheter to use a precise point collision detection or not
  * The default value is false.
+ * @ingroup Evas_Object_Group_Extras
  */
 EAPI void
-evas_object_precise_is_inside_set(Evas_Object *obj, Evas_Bool precise)
+evas_object_precise_is_inside_set(Evas_Object *obj, Eina_Bool precise)
 {
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
@@ -1604,10 +1695,12 @@ evas_object_precise_is_inside_set(Evas_Object *obj, Evas_Bool precise)
 }
 
 /**
- * Determine whether an object is set to use a precise point collision detection.
+ * Determine whether an object is set to use a precise point collision
+ * detection.
  * @param obj The given object.
+ * @ingroup Evas_Object_Group_Extras
  */
-EAPI Evas_Bool
+EAPI Eina_Bool
 evas_object_precise_is_inside_get(const Evas_Object *obj)
 {
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
@@ -1615,7 +1708,3 @@ evas_object_precise_is_inside_get(const Evas_Object *obj)
    MAGIC_CHECK_END();
    return obj->precise_is_inside;
 }
-
-/**
- * @}
- */
