@@ -5,84 +5,29 @@
 #define _GNU_SOURCE
 #endif
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#include <Eina.h>
+#include <eina_safety_checks.h>
 #include "Evas.h"
 
-/* the evas module api */
-/***********************/
-/* the module api version */
-#define EVAS_MODULE_API_VERSION 1
-
-/* the module types */
-typedef enum _Evas_Module_Type
-{
-   EVAS_MODULE_TYPE_ENGINE,
-   EVAS_MODULE_TYPE_IMAGE_LOADER,
-   EVAS_MODULE_TYPE_IMAGE_SAVER,
-     EVAS_MODULE_TYPE_OBJECT
-} Evas_Module_Type;
-
-/* the module api structure, all modules should define this struct */
-typedef struct _Evas_Module_Api Evas_Module_Api;
-struct _Evas_Module_Api
-{
-   int			version;
-   Evas_Module_Type	type;
-   const char		*name;
-   const char		*author;
-};
-
-/* the module structure */
-typedef struct _Evas_Module Evas_Module;
-struct _Evas_Module
-{
-   Evas_Module_Api	*api;
-   void			*handle;	/* the dlopen handle */
-   char			*path;		/* the path where this modules is */
-   char			*name;		/* the name of the dir where this module is */
-   struct
-     {
-	int (*open)(Evas_Module *);
-	void (*close)(Evas_Module *);
-     } func;
-   void		*functions;	/* this are the functions exported by the module */
-   void		*data;		/* some internal data for the module i.e the id for engines */
-
-   Evas_Module_Type	type;		/* the type detected by the path */
-
-   int           ref; /* how many refs */
-   int           last_used; /* the cycle count when it was last used */
-
-   unsigned char	loaded : 1;
-};
-
-
-/* the internals of the module api use this struct to reference a path with a module type
- * instead of deduce the type from the path.
- * */
-typedef struct _Evas_Module_Path Evas_Module_Path;
-struct _Evas_Module_Path
-{
-   Evas_Module_Type	type;
-   char		       *path;
-};
-
-typedef struct _Evas_Module_Engine Evas_Module_Engine;
-struct _Evas_Module_Engine
-{
-   int			id;
-};
-
-/* end of evas module api */
-/**************************/
+#include "../file/evas_module.h"
+#include "../file/evas_path.h"
 
+#ifdef EVAS_MAGIC_DEBUG
 /* complain when peole pass in wrong object types etc. */
-#define MAGIC_DEBUG
+# define MAGIC_DEBUG
+#endif
 
 #define RENDER_METHOD_INVALID            0x00000000
 
 typedef struct _Evas_Layer                  Evas_Layer;
 typedef struct _Evas_Size                   Evas_Size;
 typedef struct _Evas_Aspect                 Evas_Aspect;
+typedef struct _Evas_Border                 Evas_Border;
+typedef struct _Evas_Double_Pair            Evas_Double_Pair;
 typedef struct _Evas_Size_Hints             Evas_Size_Hints;
 typedef struct _Evas_Font_Dir               Evas_Font_Dir;
 typedef struct _Evas_Font                   Evas_Font;
@@ -103,42 +48,45 @@ typedef struct _Evas_Intercept_Func_Color   Evas_Intercept_Func_Color;
 typedef struct _Evas_Key_Grab               Evas_Key_Grab;
 typedef struct _Evas_Callbacks              Evas_Callbacks;
 typedef struct _Evas_Format                 Evas_Format;
+typedef struct _Evas_Map_Point              Evas_Map_Point;
+typedef struct _Evas_Smart_Cb_Description_Array Evas_Smart_Cb_Description_Array;
+typedef struct _Evas_Post_Callback          Evas_Post_Callback;
 
-#define MAGIC_EVAS          0x70777770
-#define MAGIC_OBJ           0x71777770
-#define MAGIC_OBJ_RECTANGLE 0x71777771
-#define MAGIC_OBJ_LINE      0x71777772
-#define MAGIC_OBJ_GRADIENT  0x71777773
-#define MAGIC_OBJ_POLYGON   0x71777774
-#define MAGIC_OBJ_IMAGE     0x71777775
-#define MAGIC_OBJ_TEXT      0x71777776
-#define MAGIC_OBJ_SMART     0x71777777
-#define MAGIC_OBJ_TEXTBLOCK 0x71777778
-#define MAGIC_SMART         0x72777770
+#define MAGIC_EVAS                 0x70777770
+#define MAGIC_OBJ                  0x71777770
+#define MAGIC_OBJ_RECTANGLE        0x71777771
+#define MAGIC_OBJ_LINE             0x71777772
+#define MAGIC_OBJ_GRADIENT         0x71777773
+#define MAGIC_OBJ_POLYGON          0x71777774
+#define MAGIC_OBJ_IMAGE            0x71777775
+#define MAGIC_OBJ_TEXT             0x71777776
+#define MAGIC_OBJ_SMART            0x71777777
+#define MAGIC_OBJ_TEXTBLOCK        0x71777778
+#define MAGIC_SMART                0x72777770
+#define MAGIC_OBJ_GRADIENT_LINEAR  0x72777771
+#define MAGIC_OBJ_GRADIENT_RADIAL  0x72777772
+#define MAGIC_OBJ_SHAPE            0x72777773
+#define MAGIC_OBJ_CONTAINER        0x72777774
+#define MAGIC_OBJ_CUSTOM           0x72777775
 
 #ifdef MAGIC_DEBUG
-#define MAGIC_CHECK_FAILED(o, t, m) \
+# define MAGIC_CHECK_FAILED(o, t, m) \
 {evas_debug_error(); \
  if (!o) evas_debug_input_null(); \
  else if (((t *)o)->magic == 0) evas_debug_magic_null(); \
  else evas_debug_magic_wrong((m), ((t *)o)->magic); \
 }
-#else
-#define MAGIC_CHECK_FAILED(o, t, m)
-#endif
-#define MAGIC_CHECK(o, t, m) \
+# define MAGIC_CHECK(o, t, m) \
 {if ((!o) || (!(((t *)o)->magic == (m)))) { \
 MAGIC_CHECK_FAILED(o, t, m)
-#define MAGIC_CHECK_END() \
-}}
+# define MAGIC_CHECK_END() }}
+#else
+# define MAGIC_CHECK_FAILED(o, t, m)
+# define MAGIC_CHECK(o, t, m)  { if (!o) {
+# define MAGIC_CHECK_END() }}
+#endif
 
-#define NEW_RECT(_r, _x, _y, _w, _h) \
-{(_r) = malloc(sizeof(Evas_Rectangle)); \
-if (_r) \
-{ \
-   (_r)->x = (_x); (_r)->y = (_y); \
-   (_r)->w = (_w); (_r)->h = (_h); \
-}}
+#define NEW_RECT(_r, _x, _y, _w, _h) (_r) = eina_rectangle_new(_x, _y, _w, _h);
 
 #define MERR_NONE() _evas_alloc_error = EVAS_ALLOC_ERROR_NONE
 #define MERR_FATAL() _evas_alloc_error = EVAS_ALLOC_ERROR_FATAL
@@ -147,26 +95,26 @@ if (_r) \
 #define EVAS_OBJECT_IMAGE_FREE_FILE_AND_KEY(o)                              \
    if ((o)->cur.file)                                                       \
      {                                                                      \
-         evas_stringshare_del((o)->cur.file);                               \
+         eina_stringshare_del((o)->cur.file);                               \
 	 if ((o)->prev.file == (o)->cur.file)                               \
 	       (o)->prev.file = NULL;                                       \
 	 (o)->cur.file = NULL;                                              \
      }                                                                      \
    if ((o)->cur.key)                                                        \
      {                                                                      \
-         evas_stringshare_del((o)->cur.key);                                \
+         eina_stringshare_del((o)->cur.key);                                \
 	 if ((o)->prev.key == (o)->cur.key)                                 \
 	       (o)->prev.key = NULL;                                        \
 	 (o)->cur.key = NULL;                                               \
      }                                                                      \
    if ((o)->prev.file)                                                      \
      {                                                                      \
-         evas_stringshare_del((o)->prev.file);                              \
+         eina_stringshare_del((o)->prev.file);                              \
 	 (o)->prev.file = NULL;                                             \
      }                                                                      \
    if ((o)->prev.key)                                                       \
      {                                                                      \
-         evas_stringshare_del((o)->prev.key);                               \
+         eina_stringshare_del((o)->prev.key);                               \
 	 (o)->prev.key = NULL;                                              \
      }
 
@@ -227,6 +175,12 @@ struct _Evas_Intercept_Func
    Evas_Intercept_Func_Basic   clip_unset;
 };
 
+struct _Evas_Smart_Cb_Description_Array
+{
+   unsigned int                      size;
+   const Evas_Smart_Cb_Description **array;
+};
+
 struct _Evas_Smart
 {
    DATA32            magic;
@@ -235,6 +189,8 @@ struct _Evas_Smart
 
    const Evas_Smart_Class *smart_class;
 
+   Evas_Smart_Cb_Description_Array callbacks;
+
    unsigned char     delete_me : 1;
    unsigned char     class_allocated : 1;
 
@@ -258,34 +214,42 @@ struct _Evas_Lock
    Evas_Modifier_Mask mask; /* we have a max of 64 locks */
 };
 
+struct _Evas_Post_Callback
+{
+   Evas_Object               *obj;
+   Evas_Object_Event_Post_Cb  func;
+   const void                *data;
+   unsigned char              delete_me : 1;
+};
+
 struct _Evas_Callbacks
 {
-   Evas_Object_List *callbacks;
+   Eina_Inlist *callbacks;
    int               walking_list;
    unsigned char     deletions_waiting : 1;
 /*
-   Evas_Object_List *down;
-   Evas_Object_List *up;
-   Evas_Object_List *move;
-   Evas_Object_List *in;
-   Evas_Object_List *out;
-   Evas_Object_List *wheel;
-   Evas_Object_List *key_down;
-   Evas_Object_List *key_up;
-   Evas_Object_List *free;
-   Evas_Object_List *obj_focus_in;
-   Evas_Object_List *obj_focus_out;
-   Evas_Object_List *obj_show;
-   Evas_Object_List *obj_hide;
-   Evas_Object_List *obj_move;
-   Evas_Object_List *obj_resize;
-   Evas_Object_List *obj_restack;
+   Eina_Inlist *down;
+   Eina_Inlist *up;
+   Eina_Inlist *move;
+   Eina_Inlist *in;
+   Eina_Inlist *out;
+   Eina_Inlist *wheel;
+   Eina_Inlist *key_down;
+   Eina_Inlist *key_up;
+   Eina_Inlist *free;
+   Eina_Inlist *obj_focus_in;
+   Eina_Inlist *obj_focus_out;
+   Eina_Inlist *obj_show;
+   Eina_Inlist *obj_hide;
+   Eina_Inlist *obj_move;
+   Eina_Inlist *obj_resize;
+   Eina_Inlist *obj_restack;
  */
 };
 
 struct _Evas
 {
-   Evas_Object_List  _list_data;
+   EINA_INLIST;
 
    DATA32            magic;
 
@@ -298,7 +262,7 @@ struct _Evas
 ////      Evas_Coord         canvas_x, canvas_y;
 
       struct {
-	 Evas_List *in;
+	 Eina_List *in;
       } object;
 
    } pointer;
@@ -314,12 +278,12 @@ struct _Evas
       unsigned char  changed : 1;
    } output;
 
-   Evas_List        *damages;
-   Evas_List        *obscures;
+   Eina_List        *damages;
+   Eina_List        *obscures;
 
    Evas_Layer       *layers;
 
-   Evas_Hash        *name_hash;
+   Eina_Hash        *name_hash;
 
    int               output_validity;
 
@@ -339,11 +303,25 @@ struct _Evas
       int   info_magic;
    } engine;
 
+   Eina_Array     delete_objects;
+   Eina_Array     active_objects;
+   Eina_Array     restack_objects;
+   Eina_Array     render_objects;
+   Eina_Array     pending_objects;
+   Eina_Array     obscuring_objects;
+   Eina_Array     temporary_objects;
+   Eina_Array     calculate_objects;
+   Eina_Array     clip_changes;
+
+   Eina_List     *post_events; // free me on evas_free
+   
+   Evas_Callbacks *callbacks;
+
    int            delete_grabs;
    int            walking_grabs;
-   Evas_List     *grabs;
+   Eina_List     *grabs;
 
-   Evas_List     *font_path;
+   Eina_List     *font_path;
 
    Evas_Object   *focused;
    void          *attach_data;
@@ -353,13 +331,16 @@ struct _Evas
    int            last_mouse_down_counter;
    int            last_mouse_up_counter;
    Evas_Font_Hinting_Flags hinting;
-   unsigned char     changed : 1;
+   unsigned char  changed : 1;
    unsigned char  delete_me : 1;
+   unsigned char  invalidate : 1;
+   unsigned char  cleanup : 1;
+   unsigned char  focus : 1;
 };
 
 struct _Evas_Layer
 {
-   Evas_Object_List  _list_data;
+   EINA_INLIST;
 
    short             layer;
    Evas_Object      *objects;
@@ -382,15 +363,46 @@ struct _Evas_Aspect
    Evas_Size size;
 };
 
+struct _Evas_Border
+{
+   Evas_Coord l, r, t, b;
+};
+
+struct _Evas_Double_Pair
+{
+   double x, y;
+};
+
 struct _Evas_Size_Hints
 {
    Evas_Size min, max, request;
    Evas_Aspect aspect;
+   Evas_Double_Pair align, weight;
+   Evas_Border padding;
+};
+
+struct _Evas_Map_Point
+{
+   Evas_Coord x, y, z;
+   double u, v;
+   unsigned char r, g, b, a;
+};
+
+struct _Evas_Map
+{
+   int count; // num of points
+   Evas_Coord_Rectangle normal_geometry; // bounding box of map geom actually
+   void *surface; // surface holding map if needed
+   int surface_w, surface_h; // current surface w & h alloc
+   Evas_Coord mx, my; // mouse x, y after conversion to map space
+   Eina_Bool alpha : 1;
+   Eina_Bool smooth : 1;
+   Evas_Map_Point points[]; // actual points
 };
 
 struct _Evas_Object
 {
-   Evas_Object_List  _list_data;
+   EINA_INLIST;
 
    DATA32            magic;
 
@@ -408,19 +420,22 @@ struct _Evas_Object
 	 struct {
 	    int            x, y, w, h;
 	    unsigned char  r, g, b, a;
-	    Evas_Bool      visible : 1;
-	    Evas_Bool      dirty : 1;
+	    Eina_Bool      visible : 1;
+	    Eina_Bool      dirty : 1;
 	 } clip;
       } cache;
+      double scale;
+      Evas_Map *map;
       Evas_Coord_Rectangle geometry;
       struct {
 	 unsigned char  r, g, b, a;
       } color;
       Evas_Object      *clipper;
       short             layer;
-      Evas_Bool         visible : 1;
-      Evas_Bool         have_clipees : 1;
-      Evas_Bool         anti_alias : 1;
+      Eina_Bool         usemap : 1;
+      Eina_Bool         visible : 1;
+      Eina_Bool         have_clipees : 1;
+      Eina_Bool         anti_alias : 1;
       unsigned char     interpolation_color_space : 1;
       Evas_Render_Op    render_op : 4;
    } cur, prev;
@@ -430,16 +445,16 @@ struct _Evas_Object
    Evas_Intercept_Func *interceptors;
 
    struct {
-      Evas_List *elements;
+      Eina_List *elements;
    } data;
 
-   Evas_List *grabs;
+   Eina_List *grabs;
 
    Evas_Callbacks *callbacks;
 
    struct {
-      Evas_List   *clipees;
-      Evas_List   *changes;
+      Eina_List   *clipees;
+      Eina_List   *changes;
    } clip;
 
    const Evas_Object_Func *func;
@@ -457,30 +472,38 @@ struct _Evas_Object
    int                         last_mouse_up_counter;
    int                         mouse_grabbed;
 
+   int                         last_event;
+
    Evas_Object_Pointer_Mode    pointer_mode : 1;
 
-   Evas_Bool                   store : 1;
-   Evas_Bool                   pass_events : 1;
-   Evas_Bool                   parent_pass_events : 1;
-   Evas_Bool                   parent_cache_valid : 1;
-   Evas_Bool                   repeat_events : 1;
-   Evas_Bool                   restack : 1;
-   Evas_Bool                   changed : 1;
-   Evas_Bool                   mouse_in : 1;
-   Evas_Bool                   pre_render_done : 1;
-   Evas_Bool                   intercepted : 1;
-   Evas_Bool                   focused : 1;
-   Evas_Bool                   in_layer : 1;
-   Evas_Bool                   no_propagate : 1;
-   Evas_Bool                   precise_is_inside : 1;
+   Eina_Bool                   store : 1;
+   Eina_Bool                   pass_events : 1;
+   Eina_Bool                   parent_pass_events : 1;
+   Eina_Bool                   parent_cache_valid : 1;
+   Eina_Bool                   repeat_events : 1;
+   Eina_Bool                   restack : 1;
+   Eina_Bool                   changed : 1;
+   Eina_Bool                   is_active : 1;
+   
+   Eina_Bool                   render_pre : 1;
+   Eina_Bool                   rect_del : 1;
+   Eina_Bool                   mouse_in : 1;
+   Eina_Bool                   pre_render_done : 1;
+   Eina_Bool                   intercepted : 1;
+   Eina_Bool                   focused : 1;
+   Eina_Bool                   in_layer : 1;
+   Eina_Bool                   no_propagate : 1;
+   
+   Eina_Bool                   precise_is_inside : 1;
+   Eina_Bool                   havemap_parent : 1;
 
    unsigned char               delete_me;
 };
 
 struct _Evas_Func_Node
 {
-   Evas_Object_List  _list_data;
-   void (*func) (void *data, Evas *e, Evas_Object *obj, void *event_info);
+   EINA_INLIST;
+   void (*func) ();
    void *data;
    Evas_Callback_Type type;
    unsigned char delete_me : 1;
@@ -494,9 +517,9 @@ struct _Evas_Data_Node
 
 struct _Evas_Font_Dir
 {
-   Evas_Hash *lookup;
-   Evas_List *fonts;
-   Evas_List *aliases;
+   Eina_Hash *lookup;
+   Eina_List *fonts;
+   Eina_List *aliases;
    DATA64     dir_mod_time;
    DATA64     fonts_dir_mod_time;
    DATA64     fonts_alias_mod_time;
@@ -527,6 +550,10 @@ struct _Evas_Object_Func
    void (*render_pre) (Evas_Object *obj);
    void (*render_post) (Evas_Object *obj);
 
+   unsigned int  (*type_id_get) (Evas_Object *obj);
+   unsigned int  (*visual_id_get) (Evas_Object *obj);
+   void *(*engine_data_get) (Evas_Object *obj);
+
    void (*store) (Evas_Object *obj);
    void (*unstore) (Evas_Object *obj);
 
@@ -540,13 +567,20 @@ struct _Evas_Object_Func
    int  (*was_inside) (Evas_Object *obj, Evas_Coord x, Evas_Coord y);
 
    void (*coords_recalc) (Evas_Object *obj);
+   
+   void (*scale_update) (Evas_Object *obj);
+
+   int (*has_opaque_rect) (Evas_Object *obj);
+   int (*get_opaque_rect) (Evas_Object *obj, Evas_Coord *x, Evas_Coord *y, Evas_Coord *w, Evas_Coord *h);
+
+   int (*can_map) (Evas_Object *obj);
 };
 
 struct _Evas_Func
 {
    void *(*info)                           (Evas *e);
    void (*info_free)                       (Evas *e, void *info);
-   void (*setup)                           (Evas *e, void *info);
+   int  (*setup)                           (Evas *e, void *info);
 
    void (*output_free)                     (void *data);
    void (*output_resize)                   (void *data, int w, int h);
@@ -558,8 +592,10 @@ struct _Evas_Func
    void (*output_redraws_next_update_push) (void *data, void *surface, int x, int y, int w, int h);
    void (*output_flush)                    (void *data);
    void (*output_idle_flush)               (void *data);
+   void (*output_dump)                     (void *data);
 
    void *(*context_new)                    (void *data);
+   Eina_Bool (*canvas_alpha_get)           (void *data, void *context);
    void (*context_free)                    (void *data, void *context);
    void (*context_clip_set)                (void *data, void *context, int x, int y, int w, int h);
    void (*context_clip_clip)               (void *data, void *context, int x, int y, int w, int h);
@@ -585,7 +621,30 @@ struct _Evas_Func
 
    void *(*polygon_point_add)              (void *data, void *context, void *polygon, int x, int y);
    void *(*polygon_points_clear)           (void *data, void *context, void *polygon);
-   void (*polygon_draw)                    (void *data, void *context, void *surface, void *polygon);
+   void (*polygon_draw)                    (void *data, void *context, void *surface, void *polygon, int x, int y);
+
+   void (*gradient2_color_np_stop_insert)   (void *data, void *gradient, int r, int g, int b, int a, float pos);
+   void (*gradient2_clear)                  (void *data, void *gradient);
+   void (*gradient2_fill_transform_set)     (void *data, void *gradient, void *transform);
+   void (*gradient2_fill_spread_set)        (void *data, void *gradient, int spread);
+
+   void *(*gradient2_linear_new)                   (void *data);
+   void (*gradient2_linear_free)                   (void *data, void *linear_gradient);
+   void (*gradient2_linear_fill_set)               (void *data, void *linear_gradient, float x0, float y0, float x1, float y1);
+   int  (*gradient2_linear_is_opaque)              (void *data, void *context, void *linear_gradient, int x, int y, int w, int h);
+   int  (*gradient2_linear_is_visible)             (void *data, void *context, void *linear_gradient, int x, int y, int w, int h);
+   void (*gradient2_linear_render_pre)             (void *data, void *context, void *linear_gradient);
+   void (*gradient2_linear_render_post)            (void *data, void *linear_gradient);
+   void (*gradient2_linear_draw)                   (void *data, void *context, void *surface, void *linear_gradient, int x, int y, int w, int h);
+
+   void *(*gradient2_radial_new)                   (void *data);
+   void (*gradient2_radial_free)                   (void *data, void *radial_gradient);
+   void (*gradient2_radial_fill_set)               (void *data, void *radial_gradient, float cx, float cy, float rx, float ry);
+   int  (*gradient2_radial_is_opaque)              (void *data, void *context, void *radial_gradient, int x, int y, int w, int h);
+   int  (*gradient2_radial_is_visible)             (void *data, void *context, void *radial_gradient, int x, int y, int w, int h);
+   void (*gradient2_radial_render_pre)             (void *data, void *context, void *radial_gradient);
+   void (*gradient2_radial_render_post)            (void *data, void *radial_gradient);
+   void (*gradient2_radial_draw)                   (void *data, void *context, void *surface, void *radial_gradient, int x, int y, int w, int h);
 
    void *(*gradient_new)                   (void *data);
    void (*gradient_free)                   (void *data, void *gradient);
@@ -617,6 +676,8 @@ struct _Evas_Func
    void *(*image_dirty_region)             (void *data, void *image, int x, int y, int w, int h);
    void *(*image_data_get)                 (void *data, void *image, int to_write, DATA32 **image_data);
    void *(*image_data_put)                 (void *data, void *image, DATA32 *image_data);
+   void  (*image_data_preload_request)     (void *data, void *image, const void *target);
+   void  (*image_data_preload_cancel)      (void *data, void *image, const void *target);
    void *(*image_alpha_set)                (void *data, void *image, int has_alpha);
    int  (*image_alpha_get)                 (void *data, void *image);
    void *(*image_border_set)               (void *data, void *image, int l, int r, int t, int b);
@@ -661,12 +722,20 @@ struct _Evas_Func
 
 /*    void (*image_rotation_set)              (void *data, void *image); */
 
+   void (*image_scale_hint_set)            (void *data, void *image, int hint);
+   int  (*image_scale_hint_get)            (void *data, void *image);
+   int  (*font_last_up_to_pos)             (void *data, void *font, const char *text, int x, int y);
+
+   void (*image_map4_draw)                 (void *data, void *context, void *surface, void *image, RGBA_Map_Point *p, int smooth, int level);
+   void *(*image_map_surface_new)          (void *data, int w, int h, int alpha);
+   void (*image_map_surface_free)          (void *data, void *surface);
 };
 
 struct _Evas_Image_Load_Func
 {
-  int (*file_head) (RGBA_Image *im, const char *file, const char *key);
-  int (*file_data) (RGBA_Image *im, const char *file, const char *key);
+  Eina_Bool threadable;
+  Eina_Bool (*file_head) (Image_Entry *ie, const char *file, const char *key, int *error);
+  Eina_Bool (*file_data) (Image_Entry *ie, const char *file, const char *key, int *error);
 };
 
 struct _Evas_Image_Save_Func
@@ -678,49 +747,37 @@ struct _Evas_Image_Save_Func
 extern "C" {
 #endif
 
-Evas_Object *evas_object_new(void);
+Evas_Object *evas_object_new(Evas *e);
 void evas_object_free(Evas_Object *obj, int clean_layer);
 void evas_object_inject(Evas_Object *obj, Evas *e);
 void evas_object_release(Evas_Object *obj, int clean_layer);
 void evas_object_change(Evas_Object *obj);
-Evas_List *evas_object_render_pre_visible_change(Evas_List *updates, Evas_Object *obj, int is_v, int was_v);
-Evas_List *evas_object_render_pre_clipper_change(Evas_List *updates, Evas_Object *obj);
-Evas_List *evas_object_render_pre_prev_cur_add(Evas_List *updates, Evas_Object *obj);
-void evas_object_render_pre_effect_updates(Evas_List *updates, Evas_Object *obj, int is_v, int was_v);
-Evas_List * evas_rects_return_difference_rects(int x, int y, int w, int h, int xx, int yy, int ww, int hh);
-void evas_object_clip_recalc(Evas_Object *obj);
+void evas_object_clip_changes_clean(Evas_Object *obj);
+void evas_object_render_pre_visible_change(Eina_Array *rects, Evas_Object *obj, int is_v, int was_v);
+void evas_object_render_pre_clipper_change(Eina_Array *rects, Evas_Object *obj);
+void evas_object_render_pre_prev_cur_add(Eina_Array *rects, Evas_Object *obj);
+void evas_object_render_pre_effect_updates(Eina_Array *rects, Evas_Object *obj, int is_v, int was_v);
+void evas_rects_return_difference_rects(Eina_Array *rects, int x, int y, int w, int h, int xx, int yy, int ww, int hh);
+
 void evas_object_clip_dirty(Evas_Object *obj);
 void evas_object_recalc_clippees(Evas_Object *obj);
 Evas_Layer *evas_layer_new(Evas *e);
 void evas_layer_pre_free(Evas_Layer *lay);
-void evas_layer_free(Evas_Layer *lay);
+void evas_layer_free_objects(Evas_Layer *lay);
+void evas_layer_clean(Evas *e);
 Evas_Layer *evas_layer_find(Evas *e, short layer_num);
 void evas_layer_add(Evas_Layer *lay);
 void evas_layer_del(Evas_Layer *lay);
-void evas_object_coords_recalc(Evas_Object *obj);
-int evas_object_is_active(Evas_Object *obj);
-int evas_object_is_in_output_rect(Evas_Object *obj, int x, int y, int w, int h);
+
 int evas_object_was_in_output_rect(Evas_Object *obj, int x, int y, int w, int h);
-int evas_object_is_visible(Evas_Object *obj);
-int evas_object_was_visible(Evas_Object *obj);
-int evas_object_is_opaque(Evas_Object *obj);
+
 int evas_object_was_opaque(Evas_Object *obj);
 int evas_object_is_inside(Evas_Object *obj, Evas_Coord x, Evas_Coord y);
 int evas_object_was_inside(Evas_Object *obj, Evas_Coord x, Evas_Coord y);
-//void evas_object_recalc_clippees(Evas_Object *obj);
-int evas_object_clippers_is_visible(Evas_Object *obj);
 int evas_object_clippers_was_visible(Evas_Object *obj);
+void evas_event_callback_call(Evas *e, Evas_Callback_Type type, void *event_info);
 void evas_object_event_callback_call(Evas_Object *obj, Evas_Callback_Type type, void *event_info);
-int evas_event_passes_through(Evas_Object *obj);
-Evas_List *evas_event_objects_event_list(Evas *e, Evas_Object *stop, int x, int y);
-int evas_file_path_is_full_path(const char *path);
-char *evas_file_path_join(const char *path, const char *end);
-int evas_file_path_exists(const char *path);
-int evas_file_path_is_file(const char *path);
-int evas_file_path_is_dir(const char *path);
-Evas_List *evas_file_path_list(char *path, const char *match, int match_case);
-DATA64 evas_file_modified_time(const char *file);
-char *evas_file_path_resolve(const char *file);
+Eina_List *evas_event_objects_event_list(Evas *e, Evas_Object *stop, int x, int y);
 int evas_mem_free(int mem_required);
 int evas_mem_degrade(int mem_required);
 void evas_debug_error(void);
@@ -731,22 +788,33 @@ void evas_debug_generic(const char *str);
 const char *evas_debug_magic_string_get(DATA32 magic);
 void evas_object_smart_use(Evas_Smart *s);
 void evas_object_smart_unuse(Evas_Smart *s);
+void evas_smart_cb_descriptions_fix(Evas_Smart_Cb_Description_Array *a) EINA_ARG_NONNULL(1);
+Eina_Bool evas_smart_cb_descriptions_resize(Evas_Smart_Cb_Description_Array *a, unsigned int size) EINA_ARG_NONNULL(1);
+const Evas_Smart_Cb_Description *evas_smart_cb_description_find(const Evas_Smart_Cb_Description_Array *a, const char *name) EINA_ARG_NONNULL(1, 2) EINA_PURE;
+
 void evas_object_smart_del(Evas_Object *obj);
 void evas_object_smart_cleanup(Evas_Object *obj);
 void evas_object_smart_member_raise(Evas_Object *member);
 void evas_object_smart_member_lower(Evas_Object *member);
 void evas_object_smart_member_stack_above(Evas_Object *member, Evas_Object *other);
 void evas_object_smart_member_stack_below(Evas_Object *member, Evas_Object *other);
-const Evas_Object_List *evas_object_smart_members_get_direct(const Evas_Object *obj);
+const Eina_Inlist *evas_object_smart_members_get_direct(const Evas_Object *obj);
+void evas_call_smarts_calculate(Evas *e);
 void *evas_mem_calloc(int size);
+void _evas_post_event_callback_call(Evas *e);
+void _evas_post_event_callback_free(Evas *e);
+void evas_event_callback_list_post_free(Eina_Inlist **list);
 void evas_object_event_callback_all_del(Evas_Object *obj);
 void evas_object_event_callback_cleanup(Evas_Object *obj);
+void evas_event_callback_all_del(Evas *e);
+void evas_event_callback_cleanup(Evas *e);
 void evas_object_inform_call_show(Evas_Object *obj);
 void evas_object_inform_call_hide(Evas_Object *obj);
 void evas_object_inform_call_move(Evas_Object *obj);
 void evas_object_inform_call_resize(Evas_Object *obj);
 void evas_object_inform_call_restack(Evas_Object *obj);
 void evas_object_inform_call_changed_size_hints(Evas_Object *obj);
+void evas_object_inform_call_image_preloaded(Evas_Object *obj);
 void evas_object_intercept_cleanup(Evas_Object *obj);
 int evas_object_intercept_call_show(Evas_Object *obj);
 int evas_object_intercept_call_hide(Evas_Object *obj);
@@ -764,9 +832,11 @@ void evas_object_grabs_cleanup(Evas_Object *obj);
 void evas_key_grab_free(Evas_Object *obj, const char *keyname, Evas_Modifier_Mask modifiers, Evas_Modifier_Mask not_modifiers);
 void evas_font_dir_cache_free(void);
 const char *evas_font_dir_cache_find(char *dir, char *font);
-Evas_List *evas_font_dir_available_list(const Evas* evas);
-void evas_font_dir_available_list_free(Evas_List *available);
+Eina_List *evas_font_dir_available_list(const Evas* evas);
+void evas_font_dir_available_list_free(Eina_List *available);
 void evas_font_free(Evas *evas, void *font);
+void evas_fonts_zero_free(Evas *evas);
+void evas_fonts_zero_presure(Evas *evas);
 void *evas_font_load(Evas *evas, const char *name, const char *source, int size);
 void evas_font_load_hinting_set(Evas *evas, void *font, int hinting);
 void evas_object_smart_member_cache_invalidate(Evas_Object *obj);
@@ -775,6 +845,7 @@ void _evas_object_text_rehint(Evas_Object *obj);
 void _evas_object_textblock_rehint(Evas_Object *obj);
 
 extern int _evas_alloc_error;
+extern int _evas_event_counter;
 
 struct _Evas_Imaging_Image
 {
@@ -786,41 +857,34 @@ struct _Evas_Imaging_Font
    RGBA_Font *font;
 };
 
-typedef struct _Evas_Mempool Evas_Mempool;
-
-struct _Evas_Mempool
-{
-   int           item_size;
-   int           pool_size;
-   int           usage;
-   void         *first, *last;
-};
-
-void *evas_mempool_malloc(Evas_Mempool *pool, int size);
-void  evas_mempool_free(Evas_Mempool *pool, void *ptr);
-void *evas_mempool_calloc(Evas_Mempool *pool, int size);
-
-void evas_module_paths_init(void);
-void evas_module_init(void);
-Evas_Module * evas_module_find_type(Evas_Module_Type type, const char *name);
-int evas_module_load(Evas_Module *em);
-void evas_module_unload(Evas_Module *em);
-void evas_module_ref(Evas_Module *em);
-void evas_module_unref(Evas_Module *em);
-void evas_module_use(Evas_Module *em);
-void evas_module_clean(void);
-void evas_module_shutdown(void);
-
 int evas_async_events_init(void);
 int evas_async_events_shutdown(void);
+int evas_async_target_del(const void *target);
+
+void _evas_preload_thread_init(void);
+void _evas_preload_thread_shutdown(void);
+Evas_Preload_Pthread *evas_preload_thread_run(void (*func_heavy)(void *data),
+					     void (*func_end)(void *data),
+					     void (*func_cancel)(void *data),
+					     const void *data);
+Eina_Bool evas_preload_thread_cancel(Evas_Preload_Pthread *thread);
 
 void _evas_walk(Evas *e);
 void _evas_unwalk(Evas *e);
 
 EAPI int _evas_module_engine_inherit(Evas_Func *funcs, char *name);
 
+void evas_render_invalidate(Evas *e);
+void evas_render_object_recalc(Evas_Object *obj);
+
+Eina_Bool evas_map_inside_get(const Evas_Map *m, Evas_Coord x, Evas_Coord y);
+Eina_Bool evas_map_coords_get(const Evas_Map *m, Evas_Coord x, Evas_Coord y, Evas_Coord *mx, Evas_Coord *my, int grab);
+       
 #define EVAS_API_OVERRIDE(func, api, prefix) \
      (api)->func = prefix##func
+
+#include "evas_inline.x"
+
 #ifdef __cplusplus
 }
 #endif
